#define WIN32_LEAN_AND_MEAN  



#include <windows.h>   // include important windows stuff
#include <windowsx.h> 
#include <mmsystem.h>
#include <iostream.h> // include important C/C++ stuff
#include <conio.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h> 
#include <math.h>
#include <io.h>
#include <fcntl.h>
#include <time.h>		// Time routines
#include <ddraw.h>		// directX includes
#include <dsound.h>
#include <dmksctrl.h>
#include <dmusici.h>
#include <dmusicc.h>
#include <dmusicf.h>
#include <dinput.h>

#include "T3DLIB1.h"	// game library includes
#include "T3DLIB2.h"
#include "T3DLIB3.h"

#include "main.h"
#include "world.h"
#include "tank.h"
#include "rndalgor.h"
#include "weapon.h" 
#include <vector>

using namespace std;

#define TANK_WIDTH 16
#define TANK_HEIGHT 6
#define BARREL_LENGTH 12

tank::tank(){			//default constructor

}
void tank::reset() {
	pos_y = 0;
	pos_x = 0;
	vy = 5;
	pos_arrow_y = -1;//arrow y position
	vy_arrow = 3;	//velocity for dropping arrow
	drewArrow = true;
	TOP_OF_ARROW = -1;

	angle = 125;
	velocity = 75;
	currentWeapon = MISSILE;
	health = 100;
	damageDoneInTurn = 0;
	timesDamageDisplayed = 0;
	exploded = false;



}

tank::tank(int color){
	reset();
	credits = 0;
	wpnList.clear();
	tankColor = color;
	
	for(int i = MISSILE; i<NUM_WEAPONS; i++) {
		weapon newWpn((weaponType)i,0);
		wpnList.push_back(newWpn);
	}

	wpnList[MISSILE].setAmmo(50);
	wpnList[LGMISSILE].setAmmo(2);
	wpnList[DIGGER].setAmmo(2);


	BITMAP_FILE arrowFile;
	//load the bitmap file
	Load_Bitmap_File(&arrowFile, "graphics/arrow.bmp"); //load the file
	//create the bitmap image
	Create_Bitmap(&arrowImage, 0, 0,28,38,16);
	//load the bitmap file into image
	Load_Image_Bitmap16(&arrowImage, &arrowFile, 0,0, BITMAP_EXTRACT_MODE_ABS);

	Unload_Bitmap_File(&arrowFile);

	BITMAP_FILE skullFile;
	Load_Bitmap_File(&skullFile, "graphics/skull.bmp"); //load the file

		//create the bitmap image
	Create_Bitmap(&skullImage, 0,0,16,18,16);

		//load the bitmap file into image
	Load_Image_Bitmap16(&skullImage, &skullFile, 0,0, BITMAP_EXTRACT_MODE_ABS);
	Unload_Bitmap_File(&skullFile);



	BITMAP_FILE tankImgFile;

	//load a tank file
	//RED, GREEN, BLUE, YELLOW, ORANGE, GREY, WHITE, or CYAN
	switch(tankColor){
		case RED:
			Load_Bitmap_File(&tankImgFile, "graphics/tank_red.bmp");
			break;
		case GREEN:
			Load_Bitmap_File(&tankImgFile, "graphics/tank_green.bmp");
			break;
		case BLUE:
			Load_Bitmap_File(&tankImgFile, "graphics/tank_blue.bmp");
			break;
		case YELLOW:
			Load_Bitmap_File(&tankImgFile, "graphics/tank_yellow.bmp");
			break;
		case ORANGE:
			Load_Bitmap_File(&tankImgFile, "graphics/tank_orange.bmp");
			break;
		case GREY:
			Load_Bitmap_File(&tankImgFile, "graphics/tank_grey.bmp");
			break;
		case PURPLE:
			Load_Bitmap_File(&tankImgFile, "graphics/tank_purple.bmp");
			break;
		case CYAN:
			Load_Bitmap_File(&tankImgFile, "graphics/tank_cyan.bmp");
			break;
	}



	Create_Bitmap(	&tankImg,	//image pointer
					0,0,		//x, y
					16,6,		//width, height
					16			//color depth
				 );

	Load_Image_Bitmap16(&tankImg,&tankImgFile,0,0,BITMAP_EXTRACT_MODE_ABS);
	
	width = tankImg.width;
	height = tankImg.height;

	barrelVertices= new VERTEX2DF [2];
}

void tank::draw(UCHAR *dest) {

	if(health > 0){
		//tank positin
		tankImg.x=pos_x;//-int(TANK_WIDTH/2);
		tankImg.y=pos_y - (int)(TANK_HEIGHT);
		if(tankImg.x<0)
			tankImg.x=0;
		if(tankImg.y<0)
			tankImg.y=0;


		Draw_Bitmap16(&tankImg,dest,back_lpitch,RGB16Bit(255,255,255));
		barrelDraw(dest);
		if(!drewArrow)
			dropArrow();

	}


}

bool tank::drop() {
//Returns true if tank is done dropping
	if(isDead())
		return true;
	

	vy = (int)(vy * 1.3);
	if(vy > 16) //Set max drop velocity
		vy = 16;
		
	if(pos_y + vy < terrain_y[pos_x]) {
		pos_y += vy;
		if(pos_y>MAP_H)
			pos_y = MAP_H;

	}
	else
		if(pos_y + vy >= terrain_y[pos_x]) {
			pos_y = terrain_y[pos_x];
			clear_landing();
			if(pos_y>MAP_H)
				pos_y = MAP_H;
			vy = 5;
			return true;
		}
	
	return false;
}

void tank::clear_landing() {

	int level; //lowest level under the tank
	level = terrain_y[pos_x];
	for(int i=pos_x; i<=pos_x + TANK_WIDTH; i++)
		if(terrain_y[i] > level)
			level = terrain_y[i];

	for(i=pos_x; i<=pos_x + TANK_WIDTH; i++)
		terrain_y[i] = level;
	
	TOP_OF_ARROW = level - 150;
	if(TOP_OF_ARROW < 50)
		TOP_OF_ARROW = 50;


}
void tank::set_x(int x) {
	if(x >=0 && x<=MAP_W)
		pos_x = x;
}
void tank::set_y(int y) {
	if(y >=0 && y<=MAP_H)
		pos_y = y;	
}

void tank::getBarrelLocation(int &x, int &y) {
	x = pos_x + int(TANK_WIDTH/2) + BARREL_LENGTH * cos((PI/180) * (180-angle));
	y = pos_y - TANK_HEIGHT - BARREL_LENGTH * sin((PI/180) * angle );

}

void tank::barrelDraw(UCHAR *dest) {

	// define points of the barrel

  VERTEX2DF barrel_vertices[2] =  { -12, 0, 
								      0, 0, };


  
	barrel.state       = 1;   // turn it on
	barrel.num_verts   = 2;  
	barrel.x0          = pos_x+int(TANK_WIDTH/2); // position it
	barrel.y0          = pos_y-TANK_HEIGHT;
	barrel.xv          = 0;
	barrel.yv          = 0;

	switch(tankColor){ //RED, GREEN, BLUE, YELLOW, ORANGE, GREY, WHITE, CYAN
		case BLUE:
			tank_color_red = 0;
			tank_color_green = 0;
			tank_color_blue = 255;
			break;
		case YELLOW:
			tank_color_red = 255;
			tank_color_green = 255;
			tank_color_blue = 0;
			break;
		case RED:
			tank_color_red = 255;
			tank_color_green = 0;
			tank_color_blue = 0;
			break;
		case GREEN:
			tank_color_red = 0;
			tank_color_green = 255;
			tank_color_blue = 0;
			break;
		case ORANGE:
			tank_color_red = 255;
			tank_color_green = 130;
			tank_color_blue = 0;
			break;
		case GREY:
			tank_color_red = 128;
			tank_color_green = 128;
			tank_color_blue = 128;
			break;
		case PURPLE:
			tank_color_red = 128;
			tank_color_green = 0;
			tank_color_blue = 128;
			break;
		case CYAN:
			tank_color_red = 0;
			tank_color_green = 255;
			tank_color_blue = 255;
			break;
	}


	barrel.color	= RGB16Bit(tank_color_red,tank_color_green,tank_color_blue);


	barrel.vlist       = barrelVertices;

	for (int i = 0; i< barrel.num_verts; i++)
		barrel.vlist[i] = barrel_vertices[i];

	Rotate_Polygon2D_Mat(&barrel, angle);
	Draw_Polygon2D16(&barrel, dest, back_lpitch);
}


void tank::barrelLeft() {
	angle-=1;
	if(angle <=0)
		angle = 0;
	worldSounds->playSound(TURRET);
}
void tank::barrelRight() {
	angle+=1;
	if(angle>180)
		angle = 180;
	worldSounds->playSound(TURRET);
}

void tank::velocityIncrease() {
	velocity+=1;
	if(velocity>100) 
		velocity = 100;
}

void tank::velocityDecrease() {
	velocity-=1;
	if(velocity<=0) 
		velocity = 0;
}

bool tank::fireWeapon() {
	int wpnAmmo = wpnList[currentWeapon].getAmmo();
	if(wpnAmmo < 1)
		return false;
	else 
		wpnList[currentWeapon].setAmmo(wpnAmmo -1);
	return true;
}

weaponType tank::getCurrentWeapon(){
	return wpnList[currentWeapon].getWpnType();
}

void tank::setCurrentWeapon(weaponType weaponOfChoice){
	worldSounds->playSound(WEAPON_SWITCH, false);
	currentWeapon = weaponOfChoice;
}

int tank::getAngle() {
	return angle;
}
int tank::getVelocity() {
	return velocity;
}

int tank::getWidth(){
	return width;
}

int tank::getHeight(){
	return height;
}

int tank::getX(){
	return pos_x;
}

int tank::getY(){
	return pos_y;
}

int tank::getHealth(){
	return health;
}

void tank::setHealth(int health_value){
	if (health_value < 0)
		health = 0;
	else
		health = health_value;
}

void tank::damageTank(int damage){
	//Add the amount of damage done in the turn.
	damageDoneInTurn += damage; 
}

bool tank::endOfTurn() {
	//Returns true if tank is done updating after end of turn.

	if(drawTankDamage()) 
		return true;//Must re-enter function

	health -= damageDoneInTurn;
	damageDoneInTurn = 0;
	timesDamageDisplayed = 0;

	return false;
}


bool tank::drawTankDamage() {
	//Shows a number floating above each tank showing damage done	
	//Returns true: must re-enter function next frame
	//Returns false: done.
	if (damageDoneInTurn <=0 || health <= 0)
		return false;
	timesDamageDisplayed++;

	char buffer[10];
	memset(buffer,0,10);
	sprintf(buffer, "%d", -1 * damageDoneInTurn);
	Draw_Text_GDI(buffer, pos_x, pos_y-2*timesDamageDisplayed, RGB(0,0,0), lpddsback);
	Draw_Text_GDI(buffer, pos_x-1, pos_y-2*timesDamageDisplayed+1, RGB(255,255,255), lpddsback);	

	if(timesDamageDisplayed < 50)
		return true;
	else {
		timesDamageDisplayed = 0;
		return false;
	}

}

bool tank::hasExploded() {
	return exploded;
}

void tank::explode() {
	exploded = true;
}

bool tank::isDead() {
	if (health<=0)
		return true;
	else
		return false;
}

void tank::displayHeader(int x, int y) {
	char buffer[256];
	memset(buffer,0,256);
	name.copy(buffer,name.length(),0);
	Draw_Text_GDI(buffer, x, y, RGB(255,255,255), lpddsback);

	VLine16(25,35,x,RGB16Bit(tank_color_red, tank_color_green, tank_color_blue),back_buffer,back_lpitch);
	VLine16(25,35,x+50,RGB16Bit(tank_color_red, tank_color_green, tank_color_blue),back_buffer,back_lpitch);
	VLine16(26,36,x+51,RGB16Bit(0, 0, 0),back_buffer,back_lpitch);
	HLine16(x, x+50, 25, RGB16Bit(tank_color_red, tank_color_green, tank_color_blue),back_buffer,back_lpitch);
	HLine16(x, x+50, 35, RGB16Bit(tank_color_red, tank_color_green, tank_color_blue),back_buffer,back_lpitch);
	HLine16(x+1, x+51, 36, RGB16Bit(0, 0, 0),back_buffer,back_lpitch);

	for(int i=x+1; i<x+health/2; i++){
		VLine16(26,34,i,RGB16Bit(tank_color_red, tank_color_green, tank_color_blue),back_buffer,back_lpitch);
	}

	if (health <= 0){
		skullImage.x = x+18;
		skullImage.y = 23;
		Draw_Bitmap16(&skullImage,back_buffer,back_lpitch,1);
	}

}

void tank::dropArrow() {
//Returns true if arrow is done dropping
	if(! drop())
		return;
	if(pos_arrow_y <0)
		return;

	vy_arrow = (int)(vy_arrow * 1.3);
	if(vy_arrow > 16) //Set max drop velocity
		vy_arrow = 16;
	if(vy_arrow <-16)
		vy_arrow = -16;
			
	if(vy_arrow >0) {
		pos_arrow_y  += vy_arrow;
	
		if(pos_arrow_y >pos_y-arrowImage.height-25 ) {
			vy_arrow =-5;
			pos_arrow_y  = pos_y-arrowImage.height-25;
		}
		if(pos_arrow_y <TOP_OF_ARROW)
			pos_arrow_y = TOP_OF_ARROW;
		
		drawArrow(); 
	}
	else if(vy_arrow <0) {
		pos_arrow_y +=vy_arrow;
		if(pos_arrow_y <TOP_OF_ARROW){
			pos_arrow_y = TOP_OF_ARROW;
			vy_arrow = 0;
		}
		
		drawArrow(); 
	}


	else {
		pos_arrow_y = TOP_OF_ARROW;//arrow y position
		vy_arrow = 5;	//velocity for dropping arrow		
		drewArrow = true;
		return;
	}
				
}

void tank::drawArrow(){

	if(TOP_OF_ARROW == -1)
		return;
	drewArrow = false;
	if(pos_arrow_y <= 0) {
		vy_arrow =5;
		pos_arrow_y = TOP_OF_ARROW;
	}

	arrowImage.x = pos_x;
	arrowImage.y = pos_arrow_y;
	Draw_Bitmap16(&arrowImage,back_buffer,back_lpitch,1);
}