#define WIN32_LEAN_AND_MEAN  

#include <windows.h>   // include important windows stuff
#include <windowsx.h> 
#include <mmsystem.h>
#include <iostream.h> // include important C/C++ stuff
#include <conio.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h> 
#include <math.h>
#include <io.h>
#include <fcntl.h>
#include <time.h>		// Time routines
#include <ddraw.h>  // directX includes
#include <dsound.h>
#include <dmksctrl.h>
#include <dmusici.h>
#include <dmusicc.h>
#include <dmusicf.h>
#include <dinput.h>e

#include "T3DLIB1.h" // game library includes
#include "T3DLIB2.h"
#include "T3DLIB3.h"

#include "main.h"
#include "world.h"
#include "rndalgor.h"
#include "projectile.h"
#include "tank.h"
#include <string>
#include "explosion.h"

	
world::world() {} //Empty constructor

void world::create()
//Initialize the world for re-entry
{
	srand(time(0)); //Seed random num generators
	slongrand(time(NULL));

	DDraw_Init(MAP_W, MAP_H, 16, 1);
	Create_Bitmap(&background_bmp,0,0,800,600,16);
	//background_bmp.attr |= BITMAP_ATTR_LOADED;
	
	DInput_Init();// initialize directinput
	
	DInput_Init_Keyboard(); // acquire the keyboard only
		
	Build_Sin_Cos_Tables(); // build the 360 degree look ups
	
	// set clipping region
	min_clip_x = 0;
	max_clip_x = screen_width - 1;
	min_clip_y = 0;
	max_clip_y = screen_height - 1;
	
	//Necessary if recreating world to make sure that no stay instances
	//of vectors or old tanks exist.
	projList.clear();
	expList.clear();

	num_players = tankList.size();
	player_state = AIMING;
	round_over = false;
	showWeaponSelectMenu = false;

	for(int i=0; i<num_players; i++) { //For every tank

		//give the tank access to the terrain
		tankList[i].terrain_y=terrain_y;
		tankList[i].reset();
	}
	
	frames = 0;
	frameRate = -1;

		//random terrain color
	switch(RAND_RANGE(1, 3)){
	case 1://green
		terrain_color_red	= 0;
		terrain_color_green = 255;
		terrain_color_blue	= 0;
		break;
	case 2://yellow
		terrain_color_red	= 255;
		terrain_color_green = 255;
		terrain_color_blue	= 0;
		break;
	case 3://brown
		terrain_color_red	= 133;
		terrain_color_green = 97;
		terrain_color_blue	= 35;
		break;
	}

	//load the menu image
	BITMAP_FILE menuFile;
	Load_Bitmap_File(&menuFile, "graphics/weapon_select.bmp"); //load the file

		//create the bitmap image
	Create_Bitmap(&menuImage, 638,60,152,208,16);

		//load the bitmap file into image
	Load_Image_Bitmap16(&menuImage, &menuFile, 0,0, BITMAP_EXTRACT_MODE_ABS);
	Unload_Bitmap_File(&menuFile);

	DDraw_Lock_Back_Surface();
	generateTerrain();
	generateBG(); 
	current_player = RAND_RANGE(0,num_players-1);
	placeTanks();	//Set tank locations

}

world::~world() {
	// shut everything down
	
	// shutdown directdraw last
	DDraw_Shutdown();
	
	// now directsound
	DSound_Stop_All_Sounds();
	DSound_Shutdown();
	
	// shut down directinput
	DInput_Shutdown();	
}

bool world::updateScreen() {
	//Like game_main.  Runs every game tick
	if(round_over)
		return true;
	
	Start_Clock();
	frames ++;

	// clear the drawing surface
	DDraw_Fill_Surface(lpddsback, 0);
	
	// lock back buffer and copy background into it
	DDraw_Lock_Back_Surface();
	
	Draw_Bitmap16(&background_bmp, back_buffer, back_lpitch,0);
	drawTerrain(back_buffer);
	
	screenHeader();

	kbInput();	//Get keyboard input

	updateProjectiles();	//Update every projectile
	updateExplosions();		//Update every explosion

	tankList[current_player].wpnList[tankList[current_player].currentWeapon].drawIcon();	

	for(int i=0; i<num_players; i++){
		if(player_state == AIMING)
			tankList[i].drop();		//Keep trying to drop a tank
		tankList[i].draw(back_buffer);	//Draw each live tank
	}
	
	if(player_state == AIMING)
		tankList[current_player].drawArrow();

	//if the current player has no ammo at all for any weapon, go to the next player
	if(player_state == FIRED && expList.empty() && projList.empty()){
		player_state = DAMAGE_DISPLAY;
	}

	//Pop up the number above the tank revealing damage
	if(player_state == DAMAGE_DISPLAY){
		damageDisplayHandler();
	}
	
	//Tank cleanup
	if(player_state == TANK_EXPLOSIONS){
		tankExplosionHandler();
	}

	//check for game over
	if(isRoundOver()){
		roundOver();
	}

	//display the menu
	if(MOUSE_X > 650 && MOUSE_X < 698 && MOUSE_Y > 5 && MOUSE_Y < 55 && !showWeaponSelectMenu){
		showWeaponSelectMenu = true;
		worldSounds->playSound(OPEN_WPN_SEL, false);
	}

	//Weapon selection popup is open
	if(showWeaponSelectMenu == true){
		weaponSelect = showMenu();
			//Clicked in the right region
		if((buttons & MK_LBUTTON) && MOUSE_X > menuImage.x && MOUSE_X < menuImage.x+menuImage.width && MOUSE_Y > menuImage.y && MOUSE_Y < menuImage.y+menuImage.height){
			if(tankList[current_player].getCurrentWeapon() != weaponSelect){
				if(tankList[current_player].wpnList[weaponSelect].getAmmo() <=0) {
					worldSounds->playSound(NO_AMMO);
				} else {
				tankList[current_player].setCurrentWeapon(weaponSelect);
				showWeaponSelectMenu = false;
				}
			}
		}
	}
	
	if(buttons & MK_LBUTTON){
		showWeaponSelectMenu = false;
	}

	// unlock back surface
	DDraw_Unlock_Back_Surface();


	// flip the surfaces
	DDraw_Flip();

	if((Get_Clock() - start_time >= 1000) || (frameRate == -1)) {
		frameRate = frames;
		frames = 0;
		start_time = Get_Clock();
	}

	// sync to 30 fps = 1/30sec = 33 ms
	Wait_Clock(30);

	
	// return success
	return false;
	
}

void world::drawTerrain(UCHAR *dest) {
	for(int x=0; x<MAP_W; x++)
		VLine16(terrain_y[x],MAP_H,x,RGB16Bit(terrain_color_red, terrain_color_green, terrain_color_blue),dest,back_lpitch);
}

void world::generateBG(){
	if(RAND_RANGE(0,6) == 0) { //Predetermined gradients
		long R = longrand() % 2;
		int y;
		if( R == 0) 
			for(y=0; y<MAP_H; y++) 
				HLine16(0,MAP_W, y, RGB16Bit(0,int(128 * y/MAP_H),int(64 * y/MAP_H)), background_bmp.buffer, back_lpitch);
		else if(R == 1)
			for(y=0; y<MAP_H; y++) 
				HLine16(0,MAP_W, MAP_H - y, RGB16Bit(int(64 * (y)/MAP_H ),int(255 * (y/2)/MAP_H) ,0), background_bmp.buffer, back_lpitch);
	}
	else { //Use a random bitmap
		BITMAP_FILE bgImgFile;
		char fileName[20], tmp[20];
		strcpy(fileName,"backgrounds\\");
		
		strcpy(strlen(fileName) + fileName, itoa(RAND_RANGE(0,NUM_BMPS),tmp,10));
		strcpy(strlen(fileName) + fileName, ".bmp");
		
		Load_Bitmap_File(&bgImgFile, fileName); //load the file
		Load_Image_Bitmap16(&background_bmp,&bgImgFile,0,0,BITMAP_EXTRACT_MODE_ABS);

	}
}


int* world::getTerrain() {
	return terrain_y;
}

void world::placeTanks() {
	int x;
	for (int a = 0; a < num_players; a++) {
		x = (int)(2 * a * MAP_W/(2.0 * num_players - 1) + longrand()%(int)(MAP_W/(2.0 * num_players - 1+6)));
		tankList[a].set_x(x);
	}
}

void world::kbInput() {
	// read keyboard
	DInput_Read_Keyboard();
	
	if(player_state == AIMING && expList.empty() && projList.empty() && !tankList[current_player].isDead()){
		//MISSILE=0, LGMISSILE, NUKE, LGNUKE, BOUNCY, CLUSTER, MEGACLUSTER, VICKS, HERBS, LASER, DIGGER, LGDIGGER, SANDBLAST, LGSANDBLAST, AIRSTRIKE, PARTICLE};

		if(keyboard_state[DIK_1]){
			if(tankList[current_player].getCurrentWeapon() == MISSILE)
				setCurrentPlayerWeapon(LGMISSILE);
			else
				setCurrentPlayerWeapon(MISSILE);
			Wait_Clock(100);
		}
		if(keyboard_state[DIK_2]){
			setCurrentPlayerWeapon(NUKE);
			Wait_Clock(100);
		}
		if(keyboard_state[DIK_3]){
			setCurrentPlayerWeapon(LGNUKE);
			Wait_Clock(100);
		}
		if(keyboard_state[DIK_4]){
			setCurrentPlayerWeapon(DIGGER);
			Wait_Clock(100);
		}
		if(keyboard_state[DIK_5]){
			setCurrentPlayerWeapon(LGDIGGER);
			Wait_Clock(100);
		}
		if(keyboard_state[DIK_6]){
			setCurrentPlayerWeapon(CLUSTER);
			Wait_Clock(100);
		}
		if(keyboard_state[DIK_7]){
			setCurrentPlayerWeapon(MEGACLUSTER);
			Wait_Clock(100);
		}
		if(keyboard_state[DIK_8]){
			setCurrentPlayerWeapon(VICKS);
			Wait_Clock(100);
		}
		if(keyboard_state[DIK_9]){
			setCurrentPlayerWeapon(HERBS);
			Wait_Clock(100);
		}
		if(keyboard_state[DIK_0]){
			setCurrentPlayerWeapon(LASER);
			Wait_Clock(100);
		}
		
			//set velocity
		if (keyboard_state[DIK_UP]) // Increase velocity
			tankList[current_player].velocityIncrease();
		if (keyboard_state[DIK_DOWN]) // Decrease Velocity
			tankList[current_player].velocityDecrease();

			//AIM - set barrel location
		if (keyboard_state[DIK_LEFT]) // rotate left
			tankList[current_player].barrelLeft();
		if (keyboard_state[DIK_RIGHT]) // rotate right
			tankList[current_player].barrelRight();

			//FIRE!!!
		if (keyboard_state[DIK_SPACE]) {
			if(Get_Clock() - last_shot < 500)
				return;
			if(fire()){	//if the player doesn't have any ammo of current weapon, fire() returns false
				player_state = FIRED;
				last_shot = Get_Clock();
			}
		}

	}
}

void world::generateTerrain()
{
	int a, b, g, h;
	int x = 0;
	int way = longrand()%2;
	float c,d,e,i,j;
	
	b = 0;
	c = 60.0f;
	d = 1.0f;
	h = (longrand() % 75) + 225; 
	i = ((longrand() % 14) / 10 - 0.7) / (float)h;	
	
	do { //Generate the terrain
		g = (longrand() % 50) + 15;//Overall smoothness
		e = ((longrand() % 250) - c) / (float)g;//Amount of huge mountains/spikiness
		for (a = x; (a < x + g) && (a <= (int)MAP_W); a++)
		{				
			terrain_y[a] = 300 - (int)(c * cos((3.142/180) * a * d) + longrand()%3 - 1);			
			//subtract from overall height

			j = sin((3.142/180)*a*d);		  
			
			if (j<0)
				j*=-1;
			
			c += e;
			h--;
			if (h<=0)
			{
				h=(longrand()%75)+15;
				i=((longrand()%10)/10 + 0.5 - d)/(float)h;
			}
			d+=i;
		}
		x = a;
	} while(x < (int)MAP_W);
	
	smoothTerrain(0,MAP_W);
}

void world::smoothTerrain(int left, int right) {
	int x, a, g;
	for (x = /*0*/ left; x < /*(int)MAP_W*/ right; x++) //Smooth the terrain
	{
		a=0;
		g=0;
		
		for (int y = -4; y <= 4; y++) //Smoothing range
		{
			if ((x + y >= 0) && (x + y < /*(int)MAP_W*/ right))
			{
				a += terrain_y[x + y];
				g++;
			}
		}
		
		if( x > 0 && x < (int)MAP_W - 1 )
			g_nGradient[x] = (int)(terrain_y[x + 1] - terrain_y[x - 1]) / 2;
		else g_nGradient[x] = 0;
		
		terrain_y[x] = a / g;
		
		if (terrain_y[x] <= 0)
			terrain_y[x] = 1;				
	}

}

void world::screenHeader() {
	// draw the title
	
	int vel = tankList[current_player].velocity;
	int ang = 180 - tankList[current_player].angle;
	
	char buffer[256];
	sprintf(buffer, "Power: %d", vel);
	Draw_Text_GDI(buffer,MAP_W-79, 6, RGB(0,0,0), lpddsback);
	Draw_Text_GDI(buffer,MAP_W-80, 5, RGB(255,255,255), lpddsback);
	sprintf(buffer, "Angle: %d ", ang);
	Draw_Text_GDI(buffer,MAP_W-79, 21, RGB(0,0,0), lpddsback);
	Draw_Text_GDI(buffer,MAP_W-80, 20, RGB(255,255,255), lpddsback);

	sprintf(buffer, "%d", frameRate);
	Draw_Text_GDI(buffer,0, 580, RGB(255,0,0), lpddsback);

	for(int i=0; i<num_players; i++)
		tankList[i].displayHeader(5+80*i, 5);
}

void world::endOfTurn() {

	//round_over = true;
	bool foundNextPlayer = false;
	bool noAmmo = false;
	int next_player = current_player;
	int i, alivePlayers=0, totalCredits = 0;

	player_state = AIMING;

		//find the next player that has health > 0
	do{
		next_player++;
		//alivePlayers = 0;
		if(next_player >= num_players)
			next_player = 0;
		if(current_player == next_player){
			//the round is over
			roundOver();

			if(noAmmo){
			//check to see if any remaining tanks have at least 100 credits left
				for (i=0; i<num_players; i++){
					if(tankList[i].credits >= 100)
						totalCredits += tankList[i].credits;
				}
				if(totalCredits == 0){
					//no tank has any ammo, or any credits to buy more
					//give each surviving tank 100 credits
					for(i=0; i<num_players; i++){
						if(!tankList[i].isDead()){
							//the tank is not dead
							tankList[i].credits += 100;
						}
					}
				}
			}
			
		}
		else{

			//check for self kill
			for(i=0; i<num_players; i++){
				if(!tankList[i].isDead())
					alivePlayers++;
			}
			if(alivePlayers < 2){
				//the person killed themselves & there is only one other player
				roundOver();
			}
		}


			
		if(tankList[next_player].isDead() && round_over != true){
			foundNextPlayer = false;
		}
		else if(round_over != true) {
			//skip the next_player if they don't have any ammo
			for(i=MISSILE; i<=LASER && !foundNextPlayer; i++){
				if(tankList[next_player].wpnList[i].getAmmo() > 0){
					foundNextPlayer = true;
					noAmmo = false;
				}
				noAmmo = true;
			}
		}
	}while(!foundNextPlayer && !round_over); //keep looping until you have found the next player

	current_player = next_player;
}
 
void world::AddProjectile(int x,int y,int vel,int ang, int owner_tank, weaponType type) {
	
	if(type == VICKS || type == HERBS){
		//get the current target (mouse location)
		int x_target = MOUSE_X;
		int y_target = MOUSE_Y;
		projectile newProj(x,y,vel,ang,owner_tank,type, x_target, y_target);
		projList.push_back(newProj);

	}
	else{
		projectile newProj(x,y,vel,ang,owner_tank,type);
		projList.push_back(newProj);
	}
	
	
}

void world::updateProjectiles() {
	
	vector<projectile>::iterator itr;			// Declare the iterators
	vector<tank>::iterator tankItr;
	vector<explosion>::iterator expItr;
	double radius, distance, expX, expY, tankX, tankY;
	int moveStatus, damage, owner;


	itr=projList.begin();
	while(itr != projList.end()) {
		//for every projectile
		if(itr->getWeaponType() !=LASER )
			moveStatus = itr->move(terrain_y); //Collision detected?
		else if(itr->getWeaponType() == LASER ){
			do{
				moveStatus = itr->move(terrain_y);
			}while(moveStatus == 0);
		}
		itr->draw(back_buffer);

		if(moveStatus == -1)
			projList.erase(itr);
			
		else if(moveStatus == 1) {
				//projectile returned collision - add an explosion
			addExplosion(itr->getWeaponType(), itr->get_x(), itr->get_y(), itr->get_radius());

				//check to see if explosion damages any tanks
			
			expItr = expList.end();
			expItr--;

			radius = expItr->getR();
			expX = expItr->getX();
			expY = expItr->getY();

			//check each tank for collision (unless the tank is already dead of course)
			tankItr = tankList.begin();
			while(tankItr != tankList.end()){
				//for every tank

				if(tankItr->getHealth() != 0){	//if the tank is alive
					tankX = tankItr->getX() + tankItr->getWidth() / 2;	//middle of the tank
					tankY = tankItr->getY() + tankItr->getHeight() / 2;

					distance = sqrt((expX - tankX)*(expX - tankX) + (expY - tankY)*(expY - tankY));

					if (distance <= radius){
						//Collision detected - damage the tank!!
							damage = itr->get_damage();
							tankItr->damageTank(damage);
							owner = itr->get_owner();
							if(tankList[owner].name != tankItr->name){
								//you hit another tank
								tankList[owner].credits += damage * 10 + 200;


							}
							else if(tankList[owner].name == tankItr->name && tankList[owner].health - tankList[owner].damageDoneInTurn <= 0){
								//you killed yourself!!!
								tankList[owner].credits -= 300;
							}

							if(tankList[owner].name != tankItr->name && tankItr->health - tankItr->damageDoneInTurn <= 0 ){
								//you destroyed another tank!!
								tankList[owner].credits += 4000;
							}
					}
				}
				tankItr++;
			}

			projList.erase(itr);
		}
		else 
			++itr;
	}

}

void world::addExplosion(weaponType wpn, int x, int y, int r) {
	explosion newExpl(terrain_y, wpn, x, y, r, current_player);
	expList.push_back(newExpl);

	switch(wpn) {
	case NUKE:
				worldSounds->playSound(NUKE_EXP,false);
				break;
	case MISSILE:
				worldSounds->playSound(MISSILE_EXP,true);
				break;
	case LGMISSILE:
				worldSounds->playSound(LGMISSILE_EXP,true);
				break;
	case LASER:
				break;
	case LGNUKE:
				worldSounds->playSound(LGNUKE_EXP,false);
				break;
	case PARTICLE: 
				break;
	default:	
				worldSounds->playSound(MISSILE_EXP,true);
				break;


	};
	
}


void world::updateExplosions() {

	int i;

	vector<explosion>::iterator itr;
	itr=expList.begin();
	while(itr != expList.end()) {

		if(itr->draw()) {
			switch (itr->getWpn()) {
			case MISSILE: 
				for (i=0; i<25; i++)		
					AddProjectile(itr->getX(), itr->getY()-10, RAND_RANGE(15,40), RAND_RANGE(60,120), itr->getPlayer(), PARTICLE); 
				
				break;
			case CLUSTER:
				for (i=0; i<50; i++)		
					AddProjectile(itr->getX(), itr->getY()-10, RAND_RANGE(15,40), RAND_RANGE(60,120), itr->getPlayer(), PARTICLE); 

				AddProjectile(itr->getX(), itr->getY()-10, RAND_RANGE(15,40), RAND_RANGE(60,120), itr->getPlayer(), MISSILE); 
				AddProjectile(itr->getX(), itr->getY()-10, RAND_RANGE(15,40), RAND_RANGE(60,120), itr->getPlayer(), MISSILE); 
				AddProjectile(itr->getX(), itr->getY()-10, RAND_RANGE(15,40), RAND_RANGE(60,120), itr->getPlayer(), MISSILE); 
				AddProjectile(itr->getX(), itr->getY()-10, RAND_RANGE(15,40), RAND_RANGE(60,120), itr->getPlayer(), MISSILE); 
				AddProjectile(itr->getX(), itr->getY()-10, RAND_RANGE(15,40), RAND_RANGE(60,120), itr->getPlayer(), MISSILE); 

				
				break;
			case LGMISSILE:
				for (i=0; i<100; i++)		
					AddProjectile(itr->getX(), itr->getY()-10, RAND_RANGE(15,40), RAND_RANGE(60,120), itr->getPlayer(), PARTICLE); 
				
				break;
			case NUKE:
				break;

			case MEGACLUSTER:
				if(expList.size() < 100)
				AddProjectile(itr->getX(), itr->getY()-10, RAND_RANGE(15,80), RAND_RANGE(60,120), itr->getPlayer(), MISSILE); 
				AddProjectile(itr->getX(), itr->getY()-10, RAND_RANGE(15,80), RAND_RANGE(60,120), itr->getPlayer(), MISSILE); 
				AddProjectile(itr->getX(), itr->getY()-10, RAND_RANGE(15,80), RAND_RANGE(60,120), itr->getPlayer(), MISSILE); 
				AddProjectile(itr->getX(), itr->getY()-10, RAND_RANGE(15,80), RAND_RANGE(60,120), itr->getPlayer(), MISSILE); 
				AddProjectile(itr->getX(), itr->getY()-10, RAND_RANGE(15,80), RAND_RANGE(60,120), itr->getPlayer(), MISSILE); 
				AddProjectile(itr->getX(), itr->getY()-10, RAND_RANGE(15,80), RAND_RANGE(60,120), itr->getPlayer(), MISSILE); 
				AddProjectile(itr->getX(), itr->getY()-10, RAND_RANGE(15,80), RAND_RANGE(60,120), itr->getPlayer(), MISSILE); 
				AddProjectile(itr->getX(), itr->getY()-10, RAND_RANGE(15,80), RAND_RANGE(60,120), itr->getPlayer(), MISSILE); 
				AddProjectile(itr->getX(), itr->getY()-10, RAND_RANGE(15,80), RAND_RANGE(60,120), itr->getPlayer(), MISSILE); 
				AddProjectile(itr->getX(), itr->getY()-10, RAND_RANGE(15,80), RAND_RANGE(60,120), itr->getPlayer(), MISSILE); 
				break;
				
			default:
				if(expList.size() < 100)
				break;
			};
			expList.erase(itr);
			
		}
		else 
			++itr;
	}
	

}


void world::damageDisplayHandler(){
	
	bool dropMore = false;
	for(int i=0; i<num_players; i++){
		if(! tankList[i].drop())
			dropMore= true;
	}
	if(dropMore)
		return;

	bool goNextState = true;
	for(i=0; i<num_players; i++){
		if(tankList[i].endOfTurn())
			goNextState = false;
	}

	if(goNextState)
		player_state = TANK_EXPLOSIONS;

}

void world::tankExplosionHandler(){

	for(int i=0; i<num_players; i++){
		if(!tankList[i].hasExploded() && tankList[i].isDead()){
			//explode the tank
			tankList[i].explode();

		}
	}

	if(expList.empty() && projList.empty()){
		endOfTurn();
	}

}


bool world::fire() {
		//Fire button was pressed.

		if(tankList[current_player].fireWeapon() == false) {
			worldSounds->playSound(NO_AMMO,false);
			return false;//No ammo
		}

		weaponType currentWpn = tankList[current_player].getCurrentWeapon();	
		int barrel_x, barrel_y;
		tankList[current_player].getBarrelLocation(barrel_x, barrel_y);
		
		switch(currentWpn) {
		case LASER:
			worldSounds->playSound(LASER_EXP,false);
			break;
		case LGNUKE:
			worldSounds->playSound(LGNUKE_FLY,false);		
		default:
			worldSounds->playSound(FIRE_WEAPON,false);

		};
		
			AddProjectile(barrel_x,
					  barrel_y,
					  tankList[current_player].getVelocity(),
					  tankList[current_player].getAngle(),
					  current_player,
					  tankList[current_player].getCurrentWeapon());
		return true;

}

void world::setCurrentPlayerWeapon(weaponType weapon){
	tankList[current_player].setCurrentWeapon(weapon);
}

int world::getCurrentPlayerIndex(){
	return current_player;
}

bool world::isRoundOver(){
	return round_over;
}

void world::roundOver(){

	DSound_Stop_All_Sounds();// stop all sounds

	round_over = true; 
		//load the bitmap file
	BITMAP_FILE roundOverFile;
	Load_Bitmap_File(&roundOverFile, "graphics/round_over.bmp"); //load the file

		//create the bitmap image
	BITMAP_IMAGE roundOverImage;
	Create_Bitmap(&roundOverImage, 240,150,400,50,16);

		//load the bitmap file into image
	Load_Image_Bitmap16(&roundOverImage, &roundOverFile, 0,0, BITMAP_EXTRACT_MODE_ABS);
	Draw_Bitmap16(&roundOverImage,back_buffer,back_lpitch,1);

}

weaponType world::showMenu(){
	Draw_Bitmap16(&menuImage,back_buffer,back_lpitch,1);

	int cellx[6] = {2,50,51,99,100,148};
	int celly[8] = {2,52,53,103,104,154,155,205};
	int cell[2];
	int i, j;
	bool selected = false;
	char buffer[256];
	memset(buffer,0,256);

	for(i=0; i<8 && !selected; i=i+2){
		for(j=0; j<6 && !selected; j=j+2){
				
				if(j==0 && i==0){
					sprintf(buffer, "%d", tankList[current_player].wpnList[MISSILE].getAmmo());
				}
				else if(j==2 && i==0){
					sprintf(buffer, "%d", tankList[current_player].wpnList[LGMISSILE].getAmmo());
				}
				else if(j==4 && i==0){
					sprintf(buffer, "%d", tankList[current_player].wpnList[NUKE].getAmmo());
				}
				else if(j==0 && i==2){
					sprintf(buffer, "%d", tankList[current_player].wpnList[LGNUKE].getAmmo());
				}
				else if(j==2 && i==2){
					sprintf(buffer, "%d", tankList[current_player].wpnList[DIGGER].getAmmo());
				}
				else if(j==4 && i==2){
					sprintf(buffer, "%d", tankList[current_player].wpnList[LGDIGGER].getAmmo());
				}
				else if(j==0 && i==4){
					sprintf(buffer, "%d", tankList[current_player].wpnList[CLUSTER].getAmmo());
				}
				else if(j==2 && i==4){
					sprintf(buffer, "%d", tankList[current_player].wpnList[MEGACLUSTER].getAmmo());
				}
				else if(j==4 && i==4){
					sprintf(buffer, "%d", tankList[current_player].wpnList[VICKS].getAmmo());
				}
				else if(j==0 && i==6){
					sprintf(buffer, "%d", tankList[current_player].wpnList[HERBS].getAmmo());
				}
				else if(j==2 && i==6){
					sprintf(buffer, "%d", tankList[current_player].wpnList[LASER].getAmmo());
				}

				Draw_Text_GDI(buffer, menuImage.x+cellx[j+1]-17, menuImage.y+celly[i+1]-15, RGB(255,255,255), lpddsback);
		}
	}

	for(i=0; i<8 && !selected; i=i+2){
		for(j=0; j<6 && !selected; j=j+2){
			if((MOUSE_X > menuImage.x+cellx[j] && MOUSE_X < menuImage.x+cellx[j+1]) && (MOUSE_Y > menuImage.y+celly[i] && MOUSE_Y < menuImage.y+celly[i+1])){
				VLine16(menuImage.y+celly[i],menuImage.y+celly[i+1],menuImage.x+cellx[j],RGB16Bit(0, 255, 0),back_buffer,back_lpitch);
				VLine16(menuImage.y+celly[i],menuImage.y+celly[i+1],menuImage.x+cellx[j+1],RGB16Bit(0, 255, 0),back_buffer,back_lpitch);

				HLine16(menuImage.x+cellx[j], menuImage.x+cellx[j+1], menuImage.y+celly[i], RGB16Bit(0, 255, 0),back_buffer,back_lpitch);
				HLine16(menuImage.x+cellx[j], menuImage.x+cellx[j+1], menuImage.y+celly[i+1], RGB16Bit(0, 255, 0),back_buffer,back_lpitch);

				if(j==0 && i==0){
					return MISSILE;
				}
				else if(j==2 && i==0){
					return LGMISSILE;
				}
				else if(j==4 && i==0){
					return NUKE;
				}
				else if(j==0 && i==2){
					return LGNUKE;
				}
				else if(j==2 && i==2){
					return DIGGER;
				}
				else if(j==4 && i==2){
					return LGDIGGER;
				}
				else if(j==0 && i==4){
					return CLUSTER;
				}
				else if(j==2 && i==4){
					return MEGACLUSTER;
				}
				else if(j==4 && i==4){
					return VICKS;
				}
				else if(j==0 && i==6){
					return HERBS;
				}
				else if(j==2 && i==6){
					return LASER;
				}
				else
					return tankList[current_player].getCurrentWeapon();

				selected = true;
			}		
		}
	}

}